==> fetch.php?media=snfit-2.2.2.tar.gz: snfit-2.2.2/src/hubblemodel.cc <==
#include <iostream>
#include <string>
#include <fstream>
#include <algorithm>
#include <cstdio>

#include "pathutils.h"
#include "fileutils.h"

#include "hubblemodel.h"
#include "fittedsn.h"
#include "snfitexception.h"

HubbleModel::HubbleModel() : hubbleparams("HubbleModel")
{
  cosmotype=LCDM;
  IntrinsicDispersion=0;
  lc_model=UNKNOWN;
}

int HubbleModel::NbParams()
{
 int nbparams = 0;
  for(size_t i=0;i< hubbleparams.size();i++) {
    if( !  hubbleparams[i]->isfixed) nbparams++;
  }
  return(nbparams);
}

void HubbleModel::dump() const
{
  cerr << IntrinsicDispersion << endl ;
}

double HubbleModel::ParamValue(const string &hubbleparamname) const
{
  LcParam const *param = hubbleparams.LocateParam(hubbleparamname);
  return(param->val) ;
}


void HubbleModel::FixParam(const string &hubbleparamname, double hubbleparamvalue)
{
  LcParam *param = const_cast<LcParam*>(hubbleparams.LocateOrAddParam(hubbleparamname));
  param->val = hubbleparamvalue;
  param->err = 0;
  param->step = fabs(hubbleparamvalue/10.); // just to remove warnings of Minuit 
  param->minv = hubbleparamvalue-fabs(hubbleparamvalue/2); // just to remove warnings of Minuit
  param->maxv = hubbleparamvalue+fabs(hubbleparamvalue/2); // just to remove warnings of Minuit
  param->isfixed = true;
}


void HubbleModel::ReadFixParam(const string file_name)
{
  cerr << "Reading model parameters value in : " << file_name << endl ;
  ifstream rd(file_name.c_str());
  string parname ;
  double val ;
  while (rd >> parname >> val)
    {
      if (parname == "Omegam") 
	continue ;
      if (parname == "Omegal") 
	continue ;
      cerr << "Fixing " << parname << " = " << val << endl ;
      FixParam(parname,val);
    }
  //hubbleparams.dump(cerr);
}
void HubbleModel::ReadCosmologyAndFixParam(const string file_name)
{
  cerr << "Reading model parameters value in : " << file_name << endl ;
  ifstream rd(file_name.c_str());
  string parname ;
  double val ;
  while (rd >> parname >> val)
    {
      cerr << "Fixing " << parname << " = " << val << endl ;
      FixParam(parname,val);
    }
  hubbleparams.dump(cerr);
}


void HubbleModel::SetParams(const double *p, const int nval) 
{
  /* NOT THE SAME NUMBER OF PARAMETERS IF SOME ARE FIXED */
  if(nval > int(hubbleparams.size())) {
    cerr << "FATAL ERROR in  Model2Fit::SetParams " << endl;
    cerr << hubbleparams.size() << " parameters in the model" << endl;
    cerr << nval << " parameters according to Minuit" << endl;
    abort();
  }

  for(size_t i = 0; i < hubbleparams.size(); ++i) hubbleparams[i]->val = p[i];
}

void HubbleModel::InitParams()
{

  // parameters for cosmology

  // om
  LcParam* om = hubbleparams.LocateOrAddNonConstParam("Omegam");
  if(!om->isfixed) {
    om->val = 0.3;
    om->step = 0.05;
    om->minv = -1;
    om->maxv = 2;
  }
  
  // ol
  if (cosmotype==LCDM) {
    LcParam* ol = hubbleparams.LocateOrAddNonConstParam("Omegal");
    if(!ol->isfixed) {
      ol->val = 0.7;
      ol->step = 0.05;
      ol->minv = -1;
      ol->maxv = 3;
    }
  }
  
  // w0
  if (cosmotype==FWCDM || cosmotype==FWWPCDM || cosmotype==FWWACDM ) {
    LcParam* w0 = hubbleparams.LocateOrAddNonConstParam("w0");
    if(!w0->isfixed) {
      w0->val = -1;
      w0->step = 0.1;
      w0->minv = -3;
      w0->maxv = 1;
    }
  }
  
  // w1
  if (cosmotype==FWWPCDM ) {
    LcParam* w1 = hubbleparams.LocateOrAddNonConstParam("w1");
    if(!w1->isfixed) {
      w1->val = 0;
      w1->step = 0.1;
      w1->minv = -10;
      w1->maxv = 10;
    }
  }
  
  // wa
  if (cosmotype==FWWACDM ) {
    LcParam* wa = hubbleparams.LocateOrAddNonConstParam("wa");
    if(!wa->isfixed) {
      wa->val = 0;
      wa->step = 0.1;
      wa->minv = -10;
      wa->maxv = 10;
    }
  }
   
 
}


#include "cosmology.h"

Cosmology* HubbleModel::GetCosmology() const {
  
  const LcParam *p = hubbleparams.LocateParam("Omegam");
  if(! p) {
    cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
    abort();
  } 
  double Omegam=p->val;
  // default value of other parameters
  double Omegal=1.-Omegam; // flat universe
  double w0=-1; // cosmological constant
  double w1=0;  
  
  
  switch (cosmotype) {
  case(LCDM): // Omegal is another parameter
    {
      const LcParam *p = hubbleparams.LocateParam("Omegal");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      Omegal=p->val;
    }
    break;
  case(FLCDM): // nothing to do
    break;
  case(FWCDM): // w0 is another parameter
    {
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
    }
    return new OmegamW0Cosmo(Omegam,w0); // need this cosmo for WMAP5 prior
    break;
  case(FWWPCDM): // w0 and w1 are free parameters
    {
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
      const LcParam *p2 = hubbleparams.LocateParam("w1");
      if(! p2) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
      abort();
      } 
      w1=p2->val;
    }
    break;
  case(FWWACDM): // w0 and wa are free parameters
    {
      double w0,wa;
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
      const LcParam *p2 = hubbleparams.LocateParam("wa");
      if(! p2) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
      abort();
      } 
      wa=p2->val;
      return new HuCosmo(Omegam,Omegal,w0,wa,1./0.9-1.); // attention a_n = 0.6
#warning forced a_n = 0.6 for HuCosmo here
    }
    
    
    break;
  default:
    cerr << "in HubbleModel::GetCosmology(), unknown cosmology to fit " << int(cosmotype) << endl;
    abort();
    break;
  }
  return new GeneralCosmo(Omegam,Omegal,w0,w1);
}

bool HubbleModel::SetCosmo2Fit(const string& what) {
  
  if(what=="LCDM") {
    SetCosmo2Fit(LCDM);
    return true;
  }
  if(what=="FLCDM") {
    SetCosmo2Fit(FLCDM);
    return true;
  }
  if(what=="FWCDM") {
    SetCosmo2Fit(FWCDM);
    return true;
  }
  if(what=="FWWPCDM") {
    SetCosmo2Fit(FWWPCDM);
    return true;
  }
  if(what=="FWWACDM") {
    SetCosmo2Fit(FWWACDM);
    return true;
  }
  cerr << "in HubbleModel::SetCosmo2Fit, unknown cosmology '" << what << "'" << endl;
  return false;
}

void HubbleModel::DumpCosmo2Fit(ostream &stream) const {
  stream << "@COSMOTYPE ";
  switch(cosmotype) {
  case LCDM     : stream << "LCDM"     << endl; break;
  case FLCDM    : stream << "FLCDM"    << endl; break;
  case FWCDM    : stream << "FWCDM"    << endl; break;
  case FWWPCDM  : stream << "FWWPCDM"  << endl; break;
  case FWWACDM  : stream << "FWWACDM"  << endl; break;
  default :
    cerr << "don't know this cosmo type, edit hubblemodel.h" << endl;
    stream << "UNKNOWN" << endl; break;
  }
}

void HubbleModel::DumpModel(ostream &stream) const {
  hubbleparams.dump(stream);
  DumpCosmo2Fit(stream);
  stream << "@LC_MODEL " << LcModel_to_string(lc_model) << endl;
  stream << "@IntrinsicDispersion " << IntrinsicDispersion << endl;
  stream << "@MalmquistBiasCorrection " << int(malmquist_bias_correction) << endl;
  stream << "@NumberOfGalaxyTypes " << int(number_of_galaxy_types) << endl;
}


#include "globalval.h"
void HubbleModel::ReadModel(const std::string &FileName)
{
  bool debug = true;
  FitParams params_in_file("HubbleModel");
  params_in_file.read(FileName);
  
  GlobalVal gv(FileName);
  string cosmo_type_in_file = gv.getStringValue("COSMOTYPE");
  if ( ! SetCosmo2Fit(cosmo_type_in_file)) {
    throw(SnfitException(" HubbleModel::ReadModel : missing or wrong COSMOTYPE key"));
  }
  //  if(debug) cout << "Cosmology : " << cosmo_name_in_file << endl;
  
  string lc_model_in_file = gv.getStringValue("LC_MODEL");
  if(string_to_LcModel(lc_model_in_file) != lc_model) {
    throw(SnfitException("HubbleModel::ReadModel : incompatible lc models"));
  }
  //  if(debug) cout << "LC Model  : " << lc_model_in_file << endl;

  IntrinsicDispersion = gv.getDoubleValue("IntrinsicDispersion");
  malmquist_bias_correction = (gv.getDoubleValue("MalmquistBiasCorrection")!=0);
  number_of_galaxy_types = int(gv.getDoubleValue("NumberOfGalaxyTypes"));

  //  if(debug) cout << "IntrinsicDispersion : " << IntrinsicDispersion << endl;  
  //if(debug) cout << "MalmquistBiasCorrection " << int(malmquist_bias_correction) << endl;
  //  if(debug) cout << "NumberOfGalaxyTypes " << int(number_of_galaxy_types) << endl;  
  
  InitParams(); // do it here cause it depends on the number of galaxy types
  
  hubbleparams = params_in_file;
  
  if(cosmotype!=LCDM) {
    LcParam *ol = hubbleparams.LocateParam("Omegal");
    if(ol) {
      ol->val = 1.-hubbleparams.LocateParam("Omegam")->val;
    }
  }
}

==> fetch.php?media=snfit-2.4.1.tar.gz: snfit-2.4.2/src/hubblemodel.cc <==
#include <iostream>
#include <string>
#include <fstream>
#include <algorithm>
#include <cstdio>

#include "pathutils.h"
#include "fileutils.h"

#include "hubblemodel.h"
#include "fittedsn.h"
#include "snfitexception.h"

HubbleModel::HubbleModel() : hubbleparams("HubbleModel")
{
  cosmotype=LCDM;
  IntrinsicDispersion=0;
  lc_model=UNKNOWN;
}

int HubbleModel::NbParams()
{
 int nbparams = 0;
  for(size_t i=0;i< hubbleparams.size();i++) {
    if( !  hubbleparams[i]->isfixed) nbparams++;
  }
  return(nbparams);
}

void HubbleModel::dump() const
{
  cerr << IntrinsicDispersion << endl ;
}

double HubbleModel::ParamValue(const string &hubbleparamname) const
{
  LcParam const *param = hubbleparams.LocateParam(hubbleparamname);
  return(param->val) ;
}


void HubbleModel::FixParam(const string &hubbleparamname, double hubbleparamvalue)
{
  LcParam *param = const_cast<LcParam*>(hubbleparams.LocateOrAddParam(hubbleparamname));
  param->val = hubbleparamvalue;
  param->err = 0;
  param->step = fabs(hubbleparamvalue/10.); // just to remove warnings of Minuit 
  param->minv = hubbleparamvalue-fabs(hubbleparamvalue/2); // just to remove warnings of Minuit
  param->maxv = hubbleparamvalue+fabs(hubbleparamvalue/2); // just to remove warnings of Minuit
  param->isfixed = true;
}


void HubbleModel::ReadFixParam(const string file_name)
{
  cerr << "Reading model parameters value in : " << file_name << endl ;
  ifstream rd(file_name.c_str());
  string parname ;
  double val ;
  while (rd >> parname >> val)
    {
      if (parname == "Omegam") 
	continue ;
      if (parname == "Omegal") 
	continue ;
      cerr << "Fixing " << parname << " = " << val << endl ;
      FixParam(parname,val);
    }
  //hubbleparams.dump(cerr);
}
void HubbleModel::ReadCosmologyAndFixParam(const string file_name)
{
  cerr << "Reading model parameters value in : " << file_name << endl ;
  ifstream rd(file_name.c_str());
  string parname ;
  double val ;
  while (rd >> parname >> val)
    {
      cerr << "Fixing " << parname << " = " << val << endl ;
      FixParam(parname,val);
    }
  hubbleparams.dump(cerr);
}


void HubbleModel::SetParams(const double *p, const int nval) 
{
  /* NOT THE SAME NUMBER OF PARAMETERS IF SOME ARE FIXED */
  if(nval > int(hubbleparams.size())) {
    cerr << "FATAL ERROR in  Model2Fit::SetParams " << endl;
    cerr << hubbleparams.size() << " parameters in the model" << endl;
    cerr << nval << " parameters according to Minuit" << endl;
    abort();
  }

  for(size_t i = 0; i < hubbleparams.size(); ++i) hubbleparams[i]->val = p[i];
}

void HubbleModel::InitParams()
{

  // parameters for cosmology

  // om
  LcParam* om = hubbleparams.LocateOrAddNonConstParam("Omegam");
  if(!om->isfixed) {
    om->val = 0.3;
    om->step = 0.05;
    om->minv = -1;
    om->maxv = 2;
  }
  
  // ol
  if (cosmotype==LCDM) {
    LcParam* ol = hubbleparams.LocateOrAddNonConstParam("Omegal");
    if(!ol->isfixed) {
      ol->val = 0.7;
      ol->step = 0.05;
      ol->minv = -1;
      ol->maxv = 3;
    }
  }
  
  // w0
  if (cosmotype==FWCDM || cosmotype==FWWPCDM || cosmotype==FWWACDM ) {
    LcParam* w0 = hubbleparams.LocateOrAddNonConstParam("w0");
    if(!w0->isfixed) {
      w0->val = -1;
      w0->step = 0.1;
      w0->minv = -3;
      w0->maxv = 1;
    }
  }
  
  // w1
  if (cosmotype==FWWPCDM ) {
    LcParam* w1 = hubbleparams.LocateOrAddNonConstParam("w1");
    if(!w1->isfixed) {
      w1->val = 0;
      w1->step = 0.1;
      w1->minv = -10;
      w1->maxv = 10;
    }
  }
  
  // wa
  if (cosmotype==FWWACDM ) {
    LcParam* wa = hubbleparams.LocateOrAddNonConstParam("wa");
    if(!wa->isfixed) {
      wa->val = 0;
      wa->step = 0.1;
      wa->minv = -10;
      wa->maxv = 10;
    }
  }
   
 
}


#include "cosmology.h"

Cosmology* HubbleModel::GetCosmology() const {
  
  const LcParam *p = hubbleparams.LocateParam("Omegam");
  if(! p) {
    cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
    abort();
  } 
  double Omegam=p->val;
  // default value of other parameters
  double Omegal=1.-Omegam; // flat universe
  double w0=-1; // cosmological constant
  double w1=0;  
  
  
  switch (cosmotype) {
  case(LCDM): // Omegal is another parameter
    {
      const LcParam *p = hubbleparams.LocateParam("Omegal");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      Omegal=p->val;
    }
    break;
  case(FLCDM): // nothing to do
    break;
  case(FWCDM): // w0 is another parameter
    {
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
    }
    return new OmegamW0Cosmo(Omegam,w0); // need this cosmo for WMAP5 prior
    break;
  case(FWWPCDM): // w0 and w1 are free parameters
    {
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
      const LcParam *p2 = hubbleparams.LocateParam("w1");
      if(! p2) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
      abort();
      } 
      w1=p2->val;
    }
    break;
  case(FWWACDM): // w0 and wa are free parameters
    {
      double w0,wa;
      const LcParam *p = hubbleparams.LocateParam("w0");
      if(! p) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
	abort();
      } 
      w0=p->val;
      const LcParam *p2 = hubbleparams.LocateParam("wa");
      if(! p2) {
	cerr << "in HubbleModel::GetCosmology(), cannot find parameter " << endl;
      abort();
      } 
      wa=p2->val;
      return new OmegamOmegaxW0WaCosmo(Omegam,Omegal,w0,wa); // attention a_n = 0.6
    }
    
    
    break;
  default:
    cerr << "in HubbleModel::GetCosmology(), unknown cosmology to fit " << int(cosmotype) << endl;
    abort();
    break;
  }
  return new GeneralCosmo(Omegam,Omegal,w0,w1);
}

bool HubbleModel::SetCosmo2Fit(const string& what) {
  
  if(what=="LCDM") {
    SetCosmo2Fit(LCDM);
    return true;
  }
  if(what=="FLCDM") {
    SetCosmo2Fit(FLCDM);
    return true;
  }
  if(what=="FWCDM") {
    SetCosmo2Fit(FWCDM);
    return true;
  }
  if(what=="FWWPCDM") {
    SetCosmo2Fit(FWWPCDM);
    return true;
  }
  if(what=="FWWACDM") {
    SetCosmo2Fit(FWWACDM);
    return true;
  }
  cerr << "in HubbleModel::SetCosmo2Fit, unknown cosmology '" << what << "'" << endl;
  return false;
}

void HubbleModel::DumpCosmo2Fit(ostream &stream) const {
  stream << "@COSMOTYPE ";
  switch(cosmotype) {
  case LCDM     : stream << "LCDM"     << endl; break;
  case FLCDM    : stream << "FLCDM"    << endl; break;
  case FWCDM    : stream << "FWCDM"    << endl; break;
  case FWWPCDM  : stream << "FWWPCDM"  << endl; break;
  case FWWACDM  : stream << "FWWACDM"  << endl; break;
  default :
    cerr << "don't know this cosmo type, edit hubblemodel.h" << endl;
    stream << "UNKNOWN" << endl; break;
  }
}

void HubbleModel::DumpModel(ostream &stream) const {
  hubbleparams.dump(stream);
  DumpCosmo2Fit(stream);
  stream << "@LC_MODEL " << LcModel_to_string(lc_model) << endl;
  stream << "@IntrinsicDispersion " << IntrinsicDispersion << endl;
  stream << "@MalmquistBiasCorrection " << int(malmquist_bias_correction) << endl;
  stream << "@NumberOfGalaxyTypes " << int(number_of_galaxy_types) << endl;
}


#include "globalval.h"
void HubbleModel::ReadModel(const std::string &FileName)
{
  bool debug = true;
  FitParams params_in_file("HubbleModel");
  try {
    params_in_file.read(FileName);
  }catch(SnfitException e) {
    e.PrintMessage(cout);
    cout << "WARNING HubbleModel::ReadModel didn't find params named 'HubbleModel' in file " << FileName << ", trying any other name" << endl;
    params_in_file = FitParams();
    params_in_file.read(FileName);
    cout << "WARNING HubbleModel::ReadModel will use params called '" << params_in_file.Name() << "'" << endl;
  }

  

  GlobalVal gv(FileName);
  string cosmo_type_in_file = gv.getStringValue("COSMOTYPE");
  if ( ! SetCosmo2Fit(cosmo_type_in_file)) {
    throw(SnfitException(" HubbleModel::ReadModel : missing or wrong COSMOTYPE key"));
  }
  //  if(debug) cout << "Cosmology : " << cosmo_name_in_file << endl;
  if(gv.HasKey("LC_MODEL")) {
      string lc_model_in_file = gv.getStringValue("LC_MODEL");
      if(string_to_LcModel(lc_model_in_file) != lc_model) {
	throw(SnfitException("HubbleModel::ReadModel : incompatible lc models"));
      }
  }else{
    cout << "WARNING HubbleModel::ReadModel didn't find gkey LC_MODEL, will assume it's ok" << endl;
  }
  //  if(debug) cout << "LC Model  : " << lc_model_in_file << endl;
  
  
  IntrinsicDispersion = gv.getDoubleValue("IntrinsicDispersion");

  malmquist_bias_correction = false;
  if(gv.HasKey("MalmquistBiasCorrection")) {
    malmquist_bias_correction = (gv.getDoubleValue("MalmquistBiasCorrection")!=0);
  }else{
    cout << "WARNING HubbleModel::ReadModel didn't find gkey MalmquistBiasCorrection, will assume it's ok" << endl;
  }
  number_of_galaxy_types = 1;
  if(gv.HasKey("NumberOfGalaxyTypes"))
    number_of_galaxy_types = int(gv.getDoubleValue("NumberOfGalaxyTypes"));
  else
    cout << "WARNING HubbleModel::ReadModel didn't find gkey NumberOfGalaxyTypes, will assume it's ok" << endl;
  //  if(debug) cout << "IntrinsicDispersion : " << IntrinsicDispersion << endl;  
  //if(debug) cout << "MalmquistBiasCorrection " << int(malmquist_bias_correction) << endl;
  //  if(debug) cout << "NumberOfGalaxyTypes " << int(number_of_galaxy_types) << endl;  
  
  InitParams(); // do it here cause it depends on the number of galaxy types
  
  hubbleparams = params_in_file;
  
  if(cosmotype!=LCDM) {
    LcParam *ol = hubbleparams.LocateParam("Omegal");
    if(ol) {
      ol->val = 1.-hubbleparams.LocateParam("Omegam")->val;
    }
  }
}
